@@ -3,8 +3,4 @@ from .base_connector import BaseStorageConnector
 from .dynamodb import DynamoDbConnector
 from .memory import MemoryStorageConnector
 
-__all__ = [
-    'DynamoDbConnector',
-    'MemoryStorageConnector',
-    'BaseStorageConnector'
-]
+__all__ = ["DynamoDbConnector", "MemoryStorageConnector", "BaseStorageConnector"]
